package cn.net.connor.createTempDrawings.utils;

import javax.swing.*;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.plaf.basic.BasicComboBoxUI;
import javax.swing.plaf.basic.BasicComboPopup;
import javax.swing.plaf.basic.ComboPopup;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;
import java.awt.*;
import java.awt.event.*;

public class JTreeComboBox extends JComboBox<Object> {
    private JTree tree;
    private DefaultTreeModel treeModel;
    private boolean suppressEvents = false;

    public JTreeComboBox(DefaultMutableTreeNode root) {
    	System.out.println("JTreeComboBox캯ãrootǷΪnull: " + (root == null));
        
        
        // ʼģ
        treeModel = new DefaultTreeModel(root);
        tree = new JTree(treeModel);
        System.out.println("ѳʼ: " + (tree != null));
        
        // ԶUI
        setUI(new TreeComboBoxUI());
        System.out.println("ԶUI");
        
        // 
        tree.setRootVisible(true);
        tree.setShowsRootHandles(true);
        tree.setCellRenderer(new TreeCellRenderer());
        
        // ѡ
        tree.getSelectionModel().addTreeSelectionListener(new TreeSelectionListener() {
            @Override
            public void valueChanged(TreeSelectionEvent e) {
                if (!suppressEvents) {
                    DefaultMutableTreeNode selectedNode = (DefaultMutableTreeNode) tree.getLastSelectedPathComponent();
                    if (selectedNode != null) {
                        setSelectedItem(selectedNode);
                        hidePopup();
                    }
                }
            }
        });
        
        // ¼
        tree.addMouseListener(new MouseAdapter() {
            @Override
            public void mousePressed(MouseEvent e) {
                TreePath path = tree.getPathForLocation(e.getX(), e.getY());
                if (path != null) {
                    tree.setSelectionPath(path);
                    DefaultMutableTreeNode node = (DefaultMutableTreeNode) path.getLastPathComponent();
                    if (node != null) {
                        if (tree.isExpanded(path)) {
                            tree.collapsePath(path);
                        } else {
                            tree.expandPath(path);
                        }
                    }
                }
            }
        });
        
        // ¼
        tree.addKeyListener(new KeyAdapter() {
            @Override
            public void keyPressed(KeyEvent e) {
                if (e.getKeyCode() == KeyEvent.VK_SPACE || e.getKeyCode() == KeyEvent.VK_ENTER) {
                    TreePath selectionPath = tree.getSelectionPath();
                    if (selectionPath != null) {
                        DefaultMutableTreeNode selectedNode = (DefaultMutableTreeNode) selectionPath.getLastPathComponent();
                        if (selectedNode != null) {
                            setSelectedItem(selectedNode);
                            hidePopup();
                            e.consume();
                        }
                    }
                } else if (e.getKeyCode() == KeyEvent.VK_ESCAPE) {
                    hidePopup();
                    e.consume();
                }
            }
        });
        
        // ʼѡڵ
        setSelectedItem(root);
        
        System.out.println("JTreeComboBoxʼ");
    }
    
    @Override
    public void setSelectedItem(Object anObject) {
        suppressEvents = true;
        super.setSelectedItem(anObject);
        suppressEvents = false;
        
        // ѡ״̬
        if (anObject instanceof DefaultMutableTreeNode) {
            DefaultMutableTreeNode node = (DefaultMutableTreeNode) anObject;
            TreePath path = new TreePath(node.getPath());
            tree.setSelectionPath(path);
        }
    }
    
    public JTree getTree() {
        return tree;
    }
    
    private class TreeComboBoxUI extends BasicComboBoxUI {
        @Override
        protected ComboPopup createPopup() {
        	System.out.println("ǷΪnull: " + (tree == null));
            return new TreeComboPopup(comboBox, tree);
        }
        
        @Override
        protected ListCellRenderer<Object> createRenderer() {
            return new ListCellRenderer<Object>() {
                private final DefaultListCellRenderer renderer = new DefaultListCellRenderer();
                
                @Override
                public Component getListCellRendererComponent(JList<?> list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
                    if (value instanceof DefaultMutableTreeNode) {
                        DefaultMutableTreeNode node = (DefaultMutableTreeNode) value;
                        value = node.getUserObject();
                    }
                    return renderer.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
                }
            };
        }
    }
    
    private static class TreeComboPopup extends BasicComboPopup {
        private JTree tree;
        
        public TreeComboPopup(JComboBox<?> comboBox, JTree tree) {
            super(comboBox);
            this.tree = tree;
            System.out.println("TreeComboPopup캯ãǷΪnull: " + (tree == null));
            removeAll();
            setLayout(new BorderLayout());
            
            // ӹ
            JScrollPane scrollPane = new JScrollPane(tree);
            scrollPane.setBorder(null);
            add(scrollPane, BorderLayout.CENTER);
            
            // õС
            Dimension preferredSize = tree.getPreferredSize();
            preferredSize.width = Math.max(preferredSize.width, comboBox.getWidth());
            preferredSize.height = Math.min(preferredSize.height, 200);
            setPreferredSize(preferredSize);
        }
        
        @Override
        protected void configureList() {
            // ʹĬϵб
        }
        
        @Override
        public void show() {
            super.show();
            // չڵ
            tree.expandPath(new TreePath(((DefaultMutableTreeNode) tree.getModel().getRoot()).getPath()));
        }
    }
    
    private static class TreeCellRenderer extends DefaultTreeCellRenderer {
        @Override
        public Component getTreeCellRendererComponent(JTree tree, Object value, boolean selected, boolean expanded, boolean leaf, int row, boolean hasFocus) {
            if (value instanceof DefaultMutableTreeNode) {
                DefaultMutableTreeNode node = (DefaultMutableTreeNode) value;
                value = node.getUserObject();
            }
            return super.getTreeCellRendererComponent(tree, value, selected, expanded, leaf, row, hasFocus);
        }
    }
}